/*
 * XMOSlisp is a tiny lisp interpreter for XMOS devices.
 * Copyright (C) 2005-2010  David Boozer
 * Copyright (C) 2009-2010  Steve Rhoads
 * Copyright (C) 2010       Ruben Vandamme
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef GARBAGE_H_
#define GARBAGE_H_

#define MAGIC 0x47

typedef struct OBJECT {
   unsigned char magic;       //Contains 0x47
   unsigned char referenced;  //Is this object referenced?
   unsigned char ptrCount;    //Number of GC pointers in object
   unsigned char index;       //Number of GC pointers that have been followed
   struct OBJECT *next;       //Next object in linked list of objects
   struct OBJECT *prev;       //Previous object in linked list of objects
} OBJECT;

typedef enum type_code {CONS, INT, SYM, PRIM, COMP, SPEC, PORT, TIMER} OBJTYPE;

// Garbage Collector version of malloc()
// ptrCount is the number of GC pointers at the start of the object
void *CollectMalloc(int size, int ptrCount);

// Garbage Collector version of free()
void CollectFree(void *ptr);

// Define another fixed root pointer
void CollectRoot(void *pRoot);

// Free any unreferenced objects
int CollectGarbage(void);

#endif /*GARBAGE_H_*/
